package com.topcoder.db.generation.xsd2ddl;

import com.topcoder.util.errorhandling.BaseException;

/**
 * The ProcessingException exception is a base exception for all the other custom
 * exceptions defined in this design. It is not directly thrown. Its purpose is
 * to allow the user to catch all custom exceptions with one catch clause instead of three.
 * 
 * The exception can store a text message and can wrap an eventual cause exception.
 *
 * Thread safety:
 * 
 * The class is immutable so it is thread safe.
 *
 * @author TCSDESIGNER
 */
public class ProcessingException extends BaseException {

    /**
     * Create a new instance with the given error message.
     *
     * The message is enforced to be completed, so that the user will benefit from meaningful messages.
     *
     * Simply calls the super constructor with the message.
     *
     * @param message the message describing the exception
     *
     * @throws IllegalArgumentException if the message is null
     */
    public ProcessingException(String message) throws IllegalArgumentException {
        super(message);
    }

    /**
     * Create a new instance with the given error message and the given cause.
     *
     * The message is enforced to be completed, so that the user will benefit from meaningful messages.
     *
     * Simply calls the super constructor with the message and cause.
     *
     * @param message the message describing the exception
     * @param cause the cause of the exception
     *
     * @throws IllegalArgumentException if the message is null or if cause is null
     */
    public ProcessingException(String message, Throwable cause) throws IllegalArgumentException {
        super(message, cause);
    }

}
