package com.topcoder.db.generation.xsd2ddl.data;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * This is a data class that models a table.
 * The table has a name, a table type, an optional primary key and zero or more foreign keys.
 * 
 * All these attributes have appropriate getters and modifiers.
 * 
 * The purpose of this class (and of the entire data class structure) is to
 * provide definition data exchange between the parsing and rendering layers
 * in a format that is independent of the concrete representation details (XSD, DDL).
 * 
 * Thread safety:
 * 
 * This is a mutable data class so it is not thread safe.
 * However, the class is used in such a way that two threads do not share an instance
 * so the class is used in a thread safe way.
 * 
 * @author TCSDESIGNER
 */
public class Table {
	
	/**
	 * The name of the table.
	 * Cannot be null, empty or all spaces (but it is null until set first).
	 * Used by its setter and getter. 
	 */
	private String tableName = null;
	
	/**
	 * The type of the table.
	 * Cannot be null (but it is null until set first).
	 * Used by its setter and getter. 
	 */
	private TableType tableType = null;
	
	/**
	 * The primary key of the table.
	 * Can be null.
	 * Used by its setter and getter. 
	 */
	private Key primaryKey = null;

	/**
     * Stores the foreign keys contained in this database.
     * It is a foreign key name (String) to Key mapping.
     * Cannot be null, have null keys or values.
     * Initialized with LinkedHashMap (because we need order).
     * Used by all the foreign key accessor methods.
     */
	private Map foreignKeys = new LinkedHashMap();
	
    /**
     * Default constructor.
     * Doesn't do anything.
     */
	public Table() {
	}

	/**
	 * Gets the table name (tableName field)
	 * 
	 * @return the table name
	 */
	public String getTableName() {
		return tableName;
	}

	/**
	 * Sets the table name (tableName field)
	 * 
	 * @param tableName the table name to set
	 * 
	 * @throws IllegalArgumentException if argument is null, empty or all spaces
	 */
	public void setTableName(String tableName) throws IllegalArgumentException {
		this.tableName = tableName;
	}
	
	/**
	 * Gets the table type (tableType field)
	 * 
	 * @return the table type
	 */
	public TableType getTableType() {
		return tableType;
	}

	/**
	 * Sets the table type (tableType field)
	 * 
	 * @param tableType the table type to set
	 * 
	 * @throws IllegalArgumentException if argument is null
	 */
	public void setTableType(TableType tableType) throws IllegalArgumentException {
		this.tableType = tableType;
	}

	/**
	 * Gets the primary key (primaryKey field)
	 * 
	 * @return the primary key
	 */
	public Key getPrimaryKey() {
		return primaryKey;
	}

	/**
	 * Sets the primary key (primaryKey field)
	 * 
	 * @param primaryKey the primary key to set (null allowed)
	 */
	public void setPrimaryKey(Key primaryKey) {
		this.primaryKey = primaryKey;
	}
	
	/**
	 * Adds a foreign key to the list of foreign keys.
	 * If a foreign key with the same name exists, nothing is done and false is returned.
	 * 
	 * Implementation details:
	 * 
	 * Checks the mapping for key.getKeyName() and adds it to the mapping is not present.
	 * 
	 * @param key the foreign key to add
	 * @return true if added, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */
	public boolean addForeignKey(Key key) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Removes a foreign key from the list of foreign keys.
	 * If the foreign key doesn't exist, nothing is done and false is returned.
	 * 
	 * Implementation details:
	 * 
	 * Delete the key.getKeyName() entry.
	 * 
	 * @param key the foreign key to remove
	 * @return true if removed, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */	
	public boolean removeForeignKey(Key key) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Clears all foreign keys.
	 * 
	 * Implementation details:
	 * 
	 * Clear the map.
	 */
	public void clearForeignKeys() {
	}
	
	/**
	 * Gets all foreign keys.
	 * 
	 * Implementation details:
	 * 
	 * Make an array from the value set and return it.
	 * 
	 * @return the contained foreign keys
	 */
	public Key[] getForeignKeys() {
		return null;
	}

	/**
	 * Gets a foreign key by name.
	 * If no such foreign key exist, null is returned
	 * 
	 * Implementation details:
	 * 
	 * Finds in the mapping the value for the name key.
	 * 
	 * @param name the foreign key name
	 * @return the foreign key with the given name or null if not found
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public Key getForeignKey(String name) throws IllegalArgumentException {
		return null;
	}

}
